"""main window"""
import os
import shutil
import gobject

import xdg.DesktopEntry

import rox
from rox import g, loading, mime
import rox.basedir
from rox.Menu import Menu, Action, ToggleItem, Separator
from autostart import *

rox.Menu.set_save_name('Autostart Manager', site='rox.sourceforge.net')
popup_menu = Menu('popup_menu', [
	Action('Remove', '_removeItem','Delete', g.STOCK_REMOVE),
	])

class MainWindow(rox.Window):
	def __init__(self):
		rox.Window.__init__(self)
		self.set_title('Autostart')
		self.set_border_width(5)
		self.set_default_size(200,200)
		sw = g.ScrolledWindow()
		sw.set_shadow_type(g.SHADOW_IN)
		sw.set_policy(g.POLICY_NEVER, g.POLICY_AUTOMATIC)
		self.add(sw)
		self.av = AutostartView()
		sw.add(self.av)
		self.show_all()
		popup_menu.attach(self, self.av)

class AutostartView(g.TreeView, loading.XDSLoader):
	def __init__(self):
		ls = g.ListStore(str, str, g.gdk.Pixbuf, bool, object)
		ls.set_sort_column_id(1, g.SORT_ASCENDING)
		g.TreeView.__init__(self, ls)
		loading.XDSLoader.__init__(self, None)
		self.set_headers_visible(False)

		self.togglerender = g.CellRendererToggle()
		self.togglerender.set_property('activatable', True)
		self.togglerender.connect( 'toggled', self._toggled_cb, ls ) 
		column = g.TreeViewColumn(None, self.togglerender)
		column.add_attribute( self.togglerender, "active", 3)
		self.append_column(column)

		column = g.TreeViewColumn(None,g.CellRendererPixbuf(), pixbuf=2)
		self.append_column(column)

		column = g.TreeViewColumn(None,g.CellRendererText(), text=0)
		column.set_sort_column_id(0)
		self.append_column(column)
		self.connect('button-press-event', self._button_press)
		self._fill()

	def _toggled_cb(self, cell, path, model):
		obj = model[path][4]
		obj.set('Hidden', model[path][3])
		# always write to config home
		fname = os.path.basename(obj.getFileName())
		wpath = os.path.join(xdg.BaseDirectory.load_first_config('autostart'), fname)
		if not os.path.exists(wpath):
			newentry = xdg.DesktopEntry.DesktopEntry(wpath)
			newentry.set('Hidden', obj.getHidden())
			newentry.write()
			model[path][4] = newentry
		else:
			obj.write()
		model[path][3] = not model[path][3]

	def xds_load_from_file(self, filepath):
		if rox.isappdir(filepath):
			dentry = makeAppdirEntry(filepath)
		elif str(mime.get_type(filepath)) == 'application/x-desktop':
			dpath = os.path.join(rox.basedir.load_first_config('autostart'),'%s' % os.path.basename(filepath))
			#os.symlink(filepath, dpath)
			shutil.copyfile(filepath, dpath)
			self._fill()
			return
		elif not os.path.isdir(filepath):
			if os.access(filepath, os.X_OK):
				dentry = makeExecutableEntry(filepath)
			else:
				rox.alert('This file can not be autostarted')
				return
		else:
			rox.alert('This file can not be autostarted')
			return
		dentry.write()
		self._fill()

	def _removeItem(self):
		sel = self.get_selection()
		(model, iter) = sel.get_selected()
		dpath = model.get_value(iter, 1)
		Name = model.get_value(iter, 0)
		if rox.confirm('Really remove "%s" from autostart?' % Name,g.STOCK_DELETE, 'Remove'):
			os.remove(dpath)
			self._fill()

	def _button_press(self, window, event):
		if event.type == g.gdk.BUTTON_PRESS:
			if event.button == 3:
				sel = self.get_selection()
				(model, iter) = sel.get_selected()
				c = popup_menu.menu.get_children()
				if iter is None:
					c[0].set_sensitive(False)
				else:
					c[0].set_sensitive(True)
				popup_menu.popup(self, event)
				return True

	def _fill(self):
		icontheme = g.icon_theme_get_default()
		model = self.get_model()
		model.clear()
		confdirs = rox.basedir.load_config_paths('autostart')
		done = []
		for confdir in confdirs:
			files = os.listdir(confdir)
			for f in files:
				fpath = os.path.join(confdir, f)
				pentry = None
				dentry = xdg.DesktopEntry.DesktopEntry(fpath)
				if os.path.basename(dentry.getFileName()) in done:
					continue
				Name = dentry.getName()
				execname = dentry.getExec()
				if Name == '':
					# Find the "parent" of this entry
					pentry = findParentEntry(dentry)
					if pentry is None:
						Name = f
					else:
						Name = pentry.getName()
				if rox.isappdir(os.path.dirname(execname)):
					appicon = dentry.getIcon()
					pb = g.gdk.pixbuf_new_from_file_at_size(appicon,16,16)
				else:
					icon_theme = g.icon_theme_get_default()
					if pentry is None:
						(root, ext) = os.path.splitext(dentry.getIcon())
					else:
						(root, ext) = os.path.splitext(pentry.getIcon())
					try:
						pb = icon_theme.load_icon(root, 16, 0)
						if pb.get_width() > 16 or pb.get_height() > 16:
							pb = pb.scale_simple(16,16,g.gdk.INTERP_BILINEAR)
					except gobject.GError, exc:
						pb = None
					
				model.append((Name, fpath, pb, not dentry.getHidden(), dentry))
				done.append(os.path.basename(dentry.getFileName()))

